﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ALPS {
  [Item("MergingReducerCalculator", "")]
  [StorableClass]
  public class MergingReducerCalculator : SingleSuccessorOperator {
    private ILookupParameter<IntValue> PopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["PopulationSize"]; }
    }
    private ILookupParameter<IntValue> LayerPopulationSizeParameter {
      get { return (ILookupParameter<IntValue>)Parameters["LayerPopulationSize"]; }
    }

    [StorableConstructor]
    private MergingReducerCalculator(bool deserializing)
      : base(deserializing) { }
    private MergingReducerCalculator(MergingReducerCalculator original, Cloner cloner)
      : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MergingReducerCalculator(this, cloner);
    }
    public MergingReducerCalculator()
      : base() {
      Parameters.Add(new LookupParameter<IntValue>("PopulationSize"));
      Parameters.Add(new LookupParameter<IntValue>("LayerPopulationSize"));
    }

    public override IOperation Apply() {
      int populationSize = PopulationSizeParameter.ActualValue.Value;
      int numSubScopes = LayerPopulationSizeParameter.ActualValue.Value;

      LayerPopulationSizeParameter.ActualValue.Value = Math.Min(populationSize, numSubScopes);

      return base.Apply();
    }
  }
}